=== added file '.bzrignore'
--- .bzrignore	1970-01-01 00:00:00 +0000
+++ .bzrignore	2008-07-24 04:18:49 +0000
@@ -0,0 +1,2 @@
+./_groupcompress_c.c
+./build

=== added file '_groupcompress_c.pyx'
--- _groupcompress_c.pyx	1970-01-01 00:00:00 +0000
+++ _groupcompress_c.pyx	2008-07-24 04:18:49 +0000
@@ -0,0 +1,83 @@
+# Copyright (C) 2008 Canonical Limited.
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published
+# by the Free Software Foundation.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+# 
+
+"""Compiled extensions for doing compression."""
+
+
+cdef class EquivalenceTable:
+    """This tracks equivalencies between lists of hashable objects.
+
+    :ivar lines: The 'static' lines that will be preserved between runs.
+    """
+
+    cdef readonly object lines
+    cdef readonly object _right_lines
+    cdef object _matching_lines
+
+    def __init__(self, lines):
+        self.lines = lines
+        self._right_lines = None
+        self._generate_matching_lines()
+
+    def _generate_matching_lines(self):
+        matches = {}
+        for idx, line in enumerate(self.lines):
+            matches.setdefault(line, []).append(idx)
+        self._matching_lines = matches
+
+    def _update_matching_lines(self, new_lines, index):
+        matches = self._matching_lines
+        start_idx = len(self.lines)
+        for idx, do_index in enumerate(index):
+            if not do_index:
+                continue
+            matches.setdefault(new_lines[idx], []).append(start_idx + idx)
+
+    def get_matches(self, line):
+        """Return the lines which match the line in right."""
+        try:
+            return self._matching_lines[line]
+        except KeyError:
+            return None
+
+    def _get_matching_lines(self):
+        """Return a dictionary showing matching lines."""
+        matching = {}
+        for line in self.lines:
+            matching[line] = self.get_matches(line)
+        return matching
+
+    def get_idx_matches(self, right_idx):
+        """Return the left lines matching the right line at the given offset."""
+        line = self._right_lines[right_idx]
+        try:
+            return self._matching_lines[line]
+        except KeyError:
+            return None
+
+    def extend_lines(self, lines, index):
+        """Add more lines to the left-lines list.
+
+        :param lines: A list of lines to add
+        :param index: A True/False for each node to define if it should be
+            indexed.
+        """
+        self._update_matching_lines(lines, index)
+        self.lines.extend(lines)
+
+    def set_right_lines(self, lines):
+        """Set the lines we will be matching against."""
+        self._right_lines = lines

=== modified file 'groupcompress.py'
--- groupcompress.py	2008-07-24 03:07:06 +0000
+++ groupcompress.py	2008-07-24 04:18:49 +0000
@@ -136,6 +136,7 @@
         pos = 0
         line_locations = self.line_locations
         line_locations.set_right_lines(lines)
+        get = line_locations._matching_lines.get
         range_len = 0
         range_start = 0
         copy_ends = None
@@ -143,6 +144,7 @@
         # insert new lines. To find reusable lines we traverse 
         while pos < len(lines):
             locations = line_locations.get_idx_matches(pos)
+            # locations = get(lines[pos])
             if locations is None:
                 if copy_ends:
                     result.append((min(copy_ends) - range_len, range_start, range_len))

=== modified file 'setup.py'
--- setup.py	2008-07-24 02:35:24 +0000
+++ setup.py	2008-07-24 04:18:49 +0000
@@ -74,7 +74,7 @@
         else:
             ext_modules.append(Extension(module_name, [c_name]))
 
-# add_pyrex_extension('_groupcompress_c')
+add_pyrex_extension('_groupcompress_c')
 
 
 if __name__ == '__main__':
@@ -91,4 +91,4 @@
           package_dir={'bzrlib.plugins.groupcompress': '.'},
           cmdclass={'build_ext': build_ext_if_possible},
           ext_modules=ext_modules,
-          }
+          )

=== modified file 'tests/test_equivalence_table.py'
--- tests/test_equivalence_table.py	2008-07-24 03:11:33 +0000
+++ tests/test_equivalence_table.py	2008-07-24 04:18:49 +0000
@@ -18,21 +18,38 @@
 
 from bzrlib.plugins.groupcompress import equivalence_table
 
+class _CompiledGroupCompress(tests.Feature):
+
+    def _probe(self):
+        try:
+            import bzrlib.plugins.groupcompress._groupcompress_c
+        except ImportError:
+            return False
+        else:
+            return True
+
+    def feature_name(self):
+        return 'bzrlib.plugins.groupcompress._groupcompress_c'
+
+CompiledGroupCompress = _CompiledGroupCompress()
+
 
 class TestEquivalenceTable(tests.TestCase):
 
+    eq_class = equivalence_table.EquivalenceTable
+
     def test_create(self):
-        eq = equivalence_table.EquivalenceTable(['a', 'b'])
+        eq = self.eq_class(['a', 'b'])
 
     def test_matching_lines(self):
         lines = ['a', 'b', 'c', 'b']
-        eq = equivalence_table.EquivalenceTable(lines)
+        eq = self.eq_class(lines)
         self.assertEqual(lines, eq.lines)
         self.assertEqual({'a': [0], 'b': [1, 3], 'c': [2]},
                          eq._get_matching_lines())
 
     def test_set_right_lines(self):
-        eq = equivalence_table.EquivalenceTable(['a', 'b', 'c', 'b'])
+        eq = self.eq_class(['a', 'b', 'c', 'b'])
         eq.set_right_lines(['f', 'b', 'b'])
         self.assertEqual(None, eq.get_idx_matches(0))
         self.assertEqual([1, 3], eq.get_idx_matches(1))
@@ -43,14 +60,14 @@
         self.assertEqual(expected_left, eq.get_matches(right))
 
     def test_get_matching(self):
-        eq = equivalence_table.EquivalenceTable(['a', 'b', 'c', 'b'])
+        eq = self.eq_class(['a', 'b', 'c', 'b'])
         self.assertGetLeftMatches([1, 3], eq, 'b')
         self.assertGetLeftMatches([2], eq, 'c')
         self.assertGetLeftMatches(None, eq, 'd')
         self.assertGetLeftMatches([2], eq, 'c')
 
     def test_extend_lines(self):
-        eq = equivalence_table.EquivalenceTable(['a', 'b', 'c', 'b'])
+        eq = self.eq_class(['a', 'b', 'c', 'b'])
         eq.extend_lines(['d', 'e', 'c'], [True, True, True])
         self.assertEqual(['a', 'b', 'c', 'b', 'd', 'e', 'c'],
                          eq.lines)
@@ -60,7 +77,7 @@
                          eq._get_matching_lines())
 
     def test_extend_lines_ignored(self):
-        eq = equivalence_table.EquivalenceTable(['a', 'b', 'c', 'b'])
+        eq = self.eq_class(['a', 'b', 'c', 'b'])
         eq.extend_lines(['d', 'e', 'c'], [False, False, True])
         self.assertEqual(['a', 'b', 'c', 'b', 'd', 'e', 'c'],
                          eq.lines)
@@ -69,6 +86,16 @@
                          eq._get_matching_lines())
 
     def test_abusive(self):
-        eq = equivalence_table.EquivalenceTable(['a']*1000)
+        eq = self.eq_class(['a']*1000)
         self.assertEqual({'a': range(1000)}, eq._get_matching_lines())
         self.assertGetLeftMatches(range(1000), eq, 'a')
+
+
+class TestCompiledEquivalenceTable(TestEquivalenceTable):
+
+    _tests_need_features = [CompiledGroupCompress]
+
+    def setUp(self):
+        super(TestCompiledEquivalenceTable, self).setUp()
+        from bzrlib.plugins.groupcompress import _groupcompress_c
+        self.eq_class = _groupcompress_c.EquivalenceTable

